//****************************************************************************
//	File Name    : 'arm_exeptions.h'
//	Title        : Generic exceptions for ARM
//	Author       : Lennart Schmidt - Copyright (C) 2010
//	Created      : 2010.01.26
//	Revised      :
//	Version      : 0.1
//	Target MCU   : Atmel ARM AT91SAM7S Series
//	Editor Tabs  : 4
//
//	NOTE: This code is currently below version 1.0, and therefore is
//		considered to be lacking in some functionality or documentation,
//		or may not be fully tested.
//		Nonetheless, you can expect most functions to work.
//****************************************************************************
//	This code is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 2 of the License, or
//	(at your option) any later version.
//
//	This code is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this code. If not, see <http://www.gnu.org/licenses/>.
//****************************************************************************


/*****************************************************************************
* Product: Generic exceptions for ARM with GNU toolset
* Date of the Last Update:  Jun 13, 2007
*
*                    Q u a n t u m     L e a P s
*                    ---------------------------
*                    innovating embedded systems
*
* Copyright (C) 2002-2007 Quantum Leaps, LLC. All rights reserved.
*
* Contact information:
* Web site: http://www.quantum-leaps.com
* e-mail:   info@quantum-leaps.com
*****************************************************************************/
#ifndef arm_exceptions_h
#define arm_exceptions_h

                                                /* interrupt locking policy */
#define ARM_INT_KEY_TYPE             int

#ifdef __thumb__

    #define ARM_INT_LOCK(key_)   ((key_) = ARM_int_lock_SYS())
    #define ARM_INT_UNLOCK(key_) (ARM_int_unlock_SYS(key_))

    extern "C" ARM_INT_KEY_TYPE ARM_int_lock_SYS(void);
    extern "C" void ARM_int_unlock_SYS(ARM_INT_KEY_TYPE key);

#else

    #define ARM_INT_LOCK(key_)   do { \
        asm("MRS %0,cpsr" : "=r" (key_)); \
        asm("MSR cpsr_c,#(0x1F | 0x80 | 0x40)"); \
    } while (0)

    #define ARM_INT_UNLOCK(key_) asm("MSR cpsr_c,%0" : : "r" (key_))

#endif


void ARM_reset(void);
void ARM_undef(void);
void ARM_swi(void);
void ARM_pAbort(void);
void ARM_dAbort(void);
void ARM_reserved(void);
void ARM_irq(void);
void ARM_fiq(void);


#endif                                                         /* arm_exc_h */

