/*
 * at91lib_uart.h
 *
 *  Created on: 02.02.2010
 *      Author: Pernicius
 */

#ifndef AT91LIB_UART_H_
#define AT91LIB_UART_H_


//#include "globals.h"
// AT91SAMT7S256 definitions
#include <AT91SAM7S256.h>
// Exact-width integer types. WG14/N843 C99 Standard
#include <stdint.h>


// UART_BAUD calculates a division rate given a baud rate
// for use with uart0Init() and uart1Init()
// example: uart0Init(UART_BAUD(115200), UART_8N1);
//
// BEWARE: additional code will be generated if 'baud' is not a constant
#define UART_BAUD(baud) (uint16_t)((F_CPU+baud*8L)/(baud*16))

// Definitions for UART mode settings (databits, parity, stopbits)
#define UART_8N1	(AT91C_US_CHRL_8_BITS | AT91C_US_PAR_NONE | AT91C_US_NBSTOP_1_BIT)
#define UART_8N2	(AT91C_US_CHRL_8_BITS | AT91C_US_PAR_NONE | AT91C_US_NBSTOP_2_BIT)
#define UART_7N1	(AT91C_US_CHRL_7_BITS | AT91C_US_PAR_NONE | AT91C_US_NBSTOP_1_BIT)
#define UART_7N2	(AT91C_US_CHRL_7_BITS | AT91C_US_PAR_NONE | AT91C_US_NBSTOP_2_BIT)

#define UART_8E1	(AT91C_US_CHRL_8_BITS | AT91C_US_PAR_EVEN | AT91C_US_NBSTOP_1_BIT)
#define UART_8E2	(AT91C_US_CHRL_8_BITS | AT91C_US_PAR_EVEN | AT91C_US_NBSTOP_2_BIT)
#define UART_7E1	(AT91C_US_CHRL_7_BITS | AT91C_US_PAR_EVEN | AT91C_US_NBSTOP_1_BIT)
#define UART_7E2	(AT91C_US_CHRL_7_BITS | AT91C_US_PAR_EVEN | AT91C_US_NBSTOP_2_BIT)

#define UART_8O1	(AT91C_US_CHRL_8_BITS | AT91C_US_PAR_ODD  | AT91C_US_NBSTOP_1_BIT)
#define UART_8O2	(AT91C_US_CHRL_8_BITS | AT91C_US_PAR_ODD  | AT91C_US_NBSTOP_2_BIT)
#define UART_7O1	(AT91C_US_CHRL_7_BITS | AT91C_US_PAR_ODD  | AT91C_US_NBSTOP_1_BIT)
#define UART_7O2	(AT91C_US_CHRL_7_BITS | AT91C_US_PAR_ODD  | AT91C_US_NBSTOP_2_BIT)

// functions

//! initializes uart
void at91_uart0_init(uint16_t bauddiv, uint32_t mode);
void at91_uart1_init(uint16_t bauddiv, uint32_t mode);
void at91_dbgu_init(uint16_t bauddiv, uint32_t mode);

//! sends a single byte over the uart
int8_t at91_uart0_putc(char data);
int8_t at91_uart1_putc(char data);
int8_t at91_dbgu_putc(char data);

//! gets a single byte from the uart
char at91_uart0_getc(void);
char at91_uart1_getc(void);
char at91_dbgu_getc(void);


#endif /* AT91LIB_UART_H_ */
