// TODO: UTF-8 string functions, UTF-8 fonts


// AT91SAMT7S256 definitions
//#include <AT91SAM7S256.h>

#include "at91lib_spi.h"
#include "systimer.h"
#include "dogm128.h"


// config
#define DISP_WIDTH		128
#define DISP_HEIGHT		64

// local defines
#define A0_COMMAND		0x10
#define A0_DATA			0x00
#define DISP_PAGES		(DISP_HEIGHT/8)
#define DISP_COLLS		DISP_WIDTH
#define SPI_DEVICE		(init_flags & 0x0000000F)
#define SPI_CLKDIV		(init_flags & 0x0000FF00)

// local functions
void dogm128_chipselect(uint8_t select);
void dogm128_updatepage(uint8_t page);

// internal vars
uint8_t volatile _dogm128_dispbuffer[DISP_PAGES*DISP_COLLS];    // framebuffer
uint8_t volatile _dogm128_pagechange;
uint32_t init_flags;                                     // copy of init_flags


//............................................................................
void dogm128_chipselect(uint8_t select) {
	if (select != 0) {
		// setup A0 mode of MISO pin
		AT91C_BASE_PIOA->PIO_PER = AT91C_PA12_MISO;
		AT91C_BASE_PIOA->PIO_OER = AT91C_PA12_MISO;
		if((select & A0_COMMAND) != 0)
			AT91C_BASE_PIOA->PIO_CODR = AT91C_PA12_MISO; // mode: command
		else
			AT91C_BASE_PIOA->PIO_SODR = AT91C_PA12_MISO; // mode: data
		// setup CS
		at91_spi_chipselect(init_flags & 0x0000000F);
	}
	else {
		// release CS
		at91_spi_chipselect(SPI_CS_OFF);
		// restore MISO mode of A0 pin
		AT91C_BASE_PIOA->PIO_ASR   = AT91C_PA12_MISO;
		AT91C_BASE_PIOA->PIO_PDR   = AT91C_PA12_MISO;
		AT91C_BASE_PIOA->PIO_PPUER = AT91C_PA12_MISO;
	}
}
//............................................................................
void dogm128_reset(void) {
	// setup portpin (PA22)
	AT91C_BASE_PIOA->PIO_PER = AT91C_PIO_PA22;
	AT91C_BASE_PIOA->PIO_OER = AT91C_PIO_PA22;

	// assert reset
	AT91C_BASE_PIOA->PIO_CODR = AT91C_PIO_PA22;

	// wait 1ms
	sleep_ms(1);

	// clear reset
	AT91C_BASE_PIOA->PIO_SODR = AT91C_PIO_PA22;

	// wait 1ms
	sleep_ms(1);
}
//............................................................................
void dogm128_backlight(uint8_t flags) {
	// TODO: dogm128_backlight
}
//............................................................................
void dogm128_init(uint32_t flags) {
	init_flags = flags;

	// config SPI device
	at91_spi_config(SPI_DEVICE, SPI_CLKDIV);

	// reset display controller
	dogm128_reset();

	// chip select
	dogm128_chipselect(1 | A0_COMMAND);

	// config display
	at91_spi_sendbyte(0x40); // Display start line 0
	if (flags & DOGM128_REVERSE) {
		at91_spi_sendbyte(0xA1); // ADC reverse
		at91_spi_sendbyte(0xC0); // Normal COM0~COM63
	}
	else {
		at91_spi_sendbyte(0xA0); // ADC normal
		at91_spi_sendbyte(0xC8); // Reverse COM63~COM0
	}
	at91_spi_sendbyte(0xA6); // Display normal $A6, reverse(inverted) $A7
	at91_spi_sendbyte(0xA2); // Set bias 1/9 (Duty 1/65)
	at91_spi_sendbyte(0x2F); // Booster, Regulator and Follower on
	at91_spi_sendbyte(0xF8); // Set internal Booster to 4x
	at91_spi_sendbyte(0x00); // - value: $00 x4 / $01 x5 / $02 x6
	at91_spi_sendbyte(0x27); // Voltage Resistor ratio
	at91_spi_sendbyte(0x81); // Contrast set
	at91_spi_sendbyte(0x16); // - value: 00..63
	at91_spi_sendbyte(0xAC); // Indicator ($AC off / $AD on)
	at91_spi_sendbyte(0x00); // - value: $00 no flashing / $01 flashing
	at91_spi_sendbyte(0xAF); // Display on

	// chip select
	dogm128_chipselect(0);

	// clear buffer and update display
	dogm128_clear();
	dogm128_update();
}
//............................................................................
void dogm128_clear(void) {
	uint8_t page;
	uint8_t coll;
	for (page = 0; page < DISP_PAGES; page++) {
		for (coll = 0; coll < DISP_COLLS; coll++) {
			_dogm128_dispbuffer[page + (coll << 3)] = 0;
		} // for (coll)
	} // for (page)
	_dogm128_pagechange = ~0;
}
//............................................................................
void dogm128_updatepage(uint8_t page) {
	uint8_t coll;
	uint8_t rev = (init_flags & DOGM128_REVERSE) ? 0 : 4;

	// setup starting column address
	dogm128_chipselect(1 | A0_COMMAND);
	at91_spi_sendbyte(0xB0 + page); // Set page address to <page>
	at91_spi_sendbyte(0x10 + 0);    // Set column address to 0 (4 MSBs)
	at91_spi_sendbyte(0x00 + rev);  // Set column address to 0 (4 LSBs)
	dogm128_chipselect(0);

	// transfer buffer to display
	dogm128_chipselect(1 | A0_DATA);
	for (coll = 0; coll < DISP_COLLS; coll++) {
		at91_spi_sendbyte(_dogm128_dispbuffer[page + (coll << 3)]);
	}
	dogm128_chipselect(0);

	_dogm128_pagechange &= ~(1 << page);
}
//............................................................................
void dogm128_update(void) {
	uint8_t page;

#if 0
	// force refresh of the whole display buffer
	page_change = ~0;
#endif

#if 1
	// mark updated lines
	static uint8_t blub = 0x00;
	for (page = 0; page < DISP_PAGES; page+=2) {
		_dogm128_dispbuffer[page + (125 << 3)] = 0xff;
		_dogm128_dispbuffer[page + (126 << 3)] = blub;
	}
	for (page = 1; page < DISP_PAGES; page+=2) {
		_dogm128_dispbuffer[page + (127 << 3)] = 0xff;
		_dogm128_dispbuffer[page + (126 << 3)] = blub;
	}
	blub++;
#endif

	for (page = 0; page < DISP_PAGES; page++) {
		if (_dogm128_pagechange & (1 << page)) {
			dogm128_updatepage(page);
		}
	}
}
