/*
 * dogm128.h
 *
 *  Created on: 03.02.2010
 *      Author: Pernicius
 */

#ifndef DOGM128_H_
#define DOGM128_H_


// Exact-width integer types. WG14/N843 C99 Standard
#include <stdint.h>


void dogm128_reset(void);
void dogm128_backlight(uint8_t flags);

void dogm128_init(uint32_t flags);
#define DOGM128_SPI_DEVICE(dev)	(dev        & 0x0000000F) // SPI device address (0..15)
#define DOGM128_SPI_CLKDIV(div)	((div << 8) & 0x0000FF00) // SPI clock divisor (1..255)
#define DOGM128_NORMAL			(0)
#define DOGM128_REVERSE			(1<<4)

void dogm128_clear(void);
void dogm128_update(void);

void dogm128_setpixel(uint16_t x, uint16_t y);
void dogm128_clearpixel(uint16_t x, uint16_t y);

void dogm128_rect(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint32_t flags);
void dogm128_circle(uint16_t x, uint16_t y, uint16_t r, uint32_t flags);
#define DOGM128_FILL_WHITE		(1<<0)
#define DOGM128_FILL_BLACK		(1<<1)
#define DOGM128_FILL_NONE		(0) // default
#define DOGM128_BORDER_WHITE	(1<<2)
#define DOGM128_BORDER_BLACK	(1<<3)
#define DOGM128_BORDER_NONE		(0) // default

#define DOGM128_DRAW_TRANSPARENT	(0) // default
#define DOGM128_DRAW_OPAQUE			(1<<0)
uint16_t dogm128_drawstring(uint16_t x, uint16_t y, const uint8_t* font, char* text, uint32_t flags);
uint16_t dogm128_drawchar(uint16_t x, uint16_t y, const uint8_t* font, char c, uint32_t flags);
uint16_t dogm128_getstringwidth(const uint8_t* font, char* text);

/*
typedef struct DOGM128_IMAGE {
	uint16_t width;
	uint16_t height;
} DOGM128_IMAGE;
void dogm128_drawimage(uint16_t x, uint16_t y, DOGM128_IMAGE* image);
*/


#endif // DOGM128_H_
