/*
 * dogm128_text.c
 *
 *  Created on: 28.02.2010
 *      Author: Pernicius
 */

#include "dogm128.h"


// config
#define DISP_WIDTH		128
#define DISP_HEIGHT		64

// local defines
#define DISP_PAGES		(DISP_HEIGHT/8)
#define DISP_COLLS		DISP_WIDTH


// internal vars
extern uint8_t volatile _dogm128_dispbuffer[DISP_PAGES*DISP_COLLS];
extern uint8_t volatile _dogm128_pagechange;


//............................................................................
uint16_t dogm128_drawstring(uint16_t x, uint16_t y, const uint8_t* font, char* text, uint32_t flags) {
	uint16_t posx=x;
	for (int i = 0; text[i] != 0x00; i++)
		posx += dogm128_drawchar(posx, y, font, text[i], flags);
	return (posx);
}
//............................................................................
uint16_t dogm128_drawchar(uint16_t x, uint16_t y, const uint8_t* font, char c, uint32_t flags) {
	uint16_t idx; // data index in font
	uint8_t ci;   // char index
	uint8_t w;    // width

	idx = 2 + 256; // set index to first char data byte
	w = font[2+c]; // char width
	for (ci = 0; ci != c && ci < 256; ci++) {
		idx += font[2 + ci] * (font[0] / 8);
	}

	// undefined char
	if (w == 0) {
		w = font[2];
		idx = 2 + 256;
	}

// TODO: cleaner and faster code ... atm its very ugly code
	uint16_t addr;
	uint8_t data;
	for (uint8_t line = 0; line < (font[0] >> 3); line++) {
		for (int p = 0; p < (y & 0x07 ? 2 : 1); p++) {

			for (ci = 0; ci != w; ci++) {
				addr = (y >> 3) + ((x + ci) << 3) + p + line;
				data = p ? (font[idx + ci + (line * w)] >> (y & 0x07))
						 : (font[idx + ci + (line * w)] << (y & 0x07));

				if (x + ci >= DISP_WIDTH)
					continue;

				if (flags & DOGM128_DRAW_OPAQUE)
					_dogm128_dispbuffer[addr] = data
					                           | ((y&0x07) ? (_dogm128_dispbuffer[addr]
					                                          & (p ? (0xff << (y & 0x07))
					                            	               : (0xff >> (y & 0x07))))
					                            	       : 0);
				else
					_dogm128_dispbuffer[addr] |= data;
			} // for(ci) char

			for (ci = w; ci != w + font[1]; ci++) {
				addr = (y >> 3) + ((x + ci) << 3) + p + line;
				data = 0x00 | ((y&0x07) ? (_dogm128_dispbuffer[addr]
                                           & (p ? (0xff << (y & 0x07))
                         	                    : (0xff >> (y & 0x07))))
                         	            : 0);


				if (flags & DOGM128_DRAW_OPAQUE)
					_dogm128_dispbuffer[addr] = data;
			} // for(ci) letter spacing

			_dogm128_pagechange |= (1 << ((y >> 3) + p + line));

		} // for(p)
	} // for(line)

	return (w + font[1]);
}
//............................................................................
uint16_t dogm128_getstringwidth(const uint8_t* font, char* text) {
	uint16_t fullwidth = 0;
	uint8_t  w;

	for (int ti = 0; text[ti] != 0; ti++) {
		w = font[2+text[ti]];
		if (w == 0)
			w = font[2];
		fullwidth += w + font[1];
	}

	return (fullwidth);
}
