/*
 * systimer.c
 *
 *  Created on: 27.02.2010
 *      Author: Pernicius
 */

// ARM exceptions, including interrupt locking
#include "arm_exceptions.h"

#include "systimer.h"


static systime_t volatile _systimer;
static uint32_t _systimer_res;


//............................................................................
uint32_t systimer_getres(void) {
	return (_systimer_res);
}
//............................................................................
void systimer_setres(uint32_t ticks_per_sec) {
	_systimer_res = ticks_per_sec;
}
//............................................................................
systime_t systimer_get(void) {
	ARM_INT_KEY_TYPE int_lock_key;
	ARM_INT_LOCK(int_lock_key);

	systime_t timerval = _systimer;

	ARM_INT_UNLOCK(int_lock_key);

	return (timerval);
}
//............................................................................
void systimer_add(uint32_t ticks) {
	ARM_INT_KEY_TYPE int_lock_key;
	ARM_INT_LOCK(int_lock_key);

	_systimer += (systime_t) ticks;

	ARM_INT_UNLOCK(int_lock_key);
}
//............................................................................
void sleep_ms(uint32_t ms) {
	// starting count value
	systime_t t_start = systimer_get();

	// counts to wait
	systime_t t_wait1 = (_systimer_res * ms) / 1000;
	systime_t t_wait2 = (_systimer_res * ms * 10) / 1000;
	systime_t t_wait = (t_wait1*10 < t_wait2) ? t_wait1+1 : t_wait1;

	// ending count value
	systime_t t_stop = t_start + t_wait;

	// waiting loop
	systime_t t_now = 0;
	while(t_stop > t_now) {
		t_now = systimer_get();
	}
}
//............................................................................
void sleep_ns(uint32_t ns) {
	// starting count value
	systime_t t_start = systimer_get();

	// counts to wait
	systime_t t_wait1 = (_systimer_res * ns) / 1000 / 1000;
	systime_t t_wait2 = (_systimer_res * ns * 10) / 1000 / 1000;
	systime_t t_wait = (t_wait1*10 < t_wait2) ? t_wait1+1 : t_wait1;

	// ending count value
	systime_t t_stop = t_start + t_wait;

	// waiting loop
	systime_t t_now = 0;
	while(t_stop > t_now) {
		t_now = systimer_get();
	}
}
